@@ -81,18 +81,6 @@ void main_saphire_job_done(int job_num, char* job_title)
     printf("%d: %s done.\n", job_num+1, job_title);
 }
 
-static int mreadline_signal()
-{
-    struct sigaction sa2;
-    memset(&sa2, 0, sizeof(sa2));
-    sa2.sa_handler = SIG_DFL;
-//    sa2.sa_flags |= SA_RESTART;
-    if(sigaction(SIGINT, &sa2, NULL) < 0) {
-        perror("sigaction2");
-        exit(1);
-    }
-}
-
 #if defined(ENABLE_READLINE)
 void sig_int_readline(int signo)
 {
@@ -308,8 +296,6 @@ int main(int argc, char* argv[])
             int rcode = saphire_load(file_name, pipeout, pipein, STDERR_FILENO);
             saphire_sweep();
             sWFd_flash(pipeout);
-            sWFd_delete(pipeout);
-            sRFd_delete(pipein);
             if(rcode == -1) {
                 fprintf(stderr, "run time err\n");
                 fprintf(stderr, "%s", string_c_str(gErrMsg));
@@ -317,10 +303,6 @@ int main(int argc, char* argv[])
             saphire_final();
             mcurses_final();
 
-            for(i=0; i<vector_size(argv2); i++) {
-                string_delete(vector_item(argv2, i));
-            }
-            vector_delete(argv2);
             CHECKML_END();
             return rcode;
         }
@@ -343,12 +325,11 @@ int main(int argc, char* argv[])
         saphire_init(kATOptC, FALSE, runtime_script, FALSE);
         sWFd* pipeout = sWFd_new(STDOUT_FILENO);
         sRFd* pipein = sRFd_new(STDIN_FILENO);
+
         int rcode = saphire_shell(send_cmd + 1, "saphire -c", pipeout, pipein, STDERR_FILENO);
         saphire_sweep();
 
         sWFd_flash(pipeout);
-        sWFd_delete(pipeout);
-        sRFd_delete(pipein);
         if(rcode < 0) { 
             fprintf(stderr, "run time err\n");
             fprintf(stderr, "%s", string_c_str(gErrMsg));
@@ -357,10 +338,6 @@ int main(int argc, char* argv[])
         saphire_final();
         mcurses_final();
 
-        for(i=0; i<vector_size(argv2); i++) {
-            string_delete(vector_item(argv2, i));
-        }
-        vector_delete(argv2);
         CHECKML_END();
         return rcode;
     }
@@ -369,10 +346,6 @@ int main(int argc, char* argv[])
 
 #if !defined(ENABLE_READLINE)
     fprintf(stderr, "if you use saphire as interactive shell, use saphiresh command instead\n");
-    for(i=0; i<vector_size(argv2); i++) {
-        string_delete(vector_item(argv2, i));
-    }
-    vector_delete(argv2);
     return 0;
 #else
     /// インタラクティブシェル開始 ///
@@ -421,9 +394,6 @@ int main(int argc, char* argv[])
             else {
                 buf = readline(string_c_str(ret));
             }
-            sRFd_delete(rfd);
-
-            string_delete(ret);
         }
         else {
             buf = readline("> ");
@@ -448,8 +418,6 @@ int main(int argc, char* argv[])
             int rcode = saphire_shell(buf, "saphire", pipeout, pipein, STDERR_FILENO);
             saphire_sweep();
             sWFd_flash(pipeout);
-            sWFd_delete(pipeout);
-            sRFd_delete(pipein);
             if(rcode == -1) {
                 /// ヒストリに追加 ///
                 add_history(buf);
@@ -495,11 +463,6 @@ int main(int argc, char* argv[])
         write_history(histfile);
     }
 
-    for(i=0; i<vector_size(argv2); i++) {
-        string_delete(vector_item(argv2, i));
-    }
-    vector_delete(argv2);
-
     /// メモリリーク検出 ///
     CHECKML_END();
 #endif
