@@ -45,7 +45,6 @@ int str_termlen_range(enum eKanjiCode code, char* mbs, int utfpos1, int utfpos2)
                 mbs2[putfpos2 - putfpos1] = 0;
 
                 int result = str_termlen(kUtf8, mbs2);
-                FREE(mbs2);
 
                 return result;
             }
@@ -73,25 +72,17 @@ BOOL is_utf8_bytes(char* mbs)
     char* mbs2 = MALLOC(size);
 
     if(mbstowcs(wcs, mbs, size) < 0) {
-        FREE(wcs);
-        FREE(mbs2);
         return FALSE;
     }
 
     if(wcstombs(mbs2, wcs, size) < 0) {
-        FREE(wcs);
-        FREE(mbs2);
         return FALSE;
     }
 
     if(strcmp(mbs, mbs2) == 0) {
-        FREE(wcs);
-        FREE(mbs2);
         return TRUE;
     }
     else {
-        FREE(wcs);
-        FREE(mbs2);
         return FALSE;
     }
 }
@@ -127,8 +118,6 @@ int str_pointer2kanjipos(enum eKanjiCode code, char* mbs, char* point)
 
         result = str_kanjilen(code, mbs2);
 
-        FREE(mbs2);
-
         return result;
     }
 }
@@ -295,11 +284,9 @@ int str_termlen(enum eKanjiCode code, char* mbs)
 
         wcs = (wchar_t*)MALLOC(sizeof(wchar_t)*((strlen(mbs)+1)*MB_CUR_MAX));
         if(mbstowcs(wcs, mbs, (strlen(mbs)+1)*MB_CUR_MAX) < 0) {
-            FREE(wcs);
             return -1;
         }
         result = wcswidth(wcs, wcslen(wcs));
-        FREE(wcs);
         if(result < 0) {
             return strlen(mbs);
         }
@@ -334,8 +321,6 @@ int str_termlen2(enum eKanjiCode code, char* mbs, int pos)
             result = str_termlen(code, mbs2);
         }
 
-        FREE(mbs2);
-
         return result;
     }
     else {
@@ -349,7 +334,7 @@ int str_termlen2(enum eKanjiCode code, char* mbs, int pos)
 #if defined(HAVE_ICONV_H)
 #include <iconv.h>
 
-#if !defined(__FREEBSD__)
+#if !defined(__FREEBSD__) && !defined(__SUNOS__)
 extern size_t iconv (iconv_t cd, char* * inbuf, size_t *inbytesleft, char* * outbuf, size_t *outbytesleft);
 #endif
 
@@ -386,7 +371,7 @@ int kanji_convert(char* input_buf, char* output_buf, size_t output_buf_size, enu
         ic = iconv_open(gIconvKanjiCodeName[output_kanji_encode_type]
                         , gIconvKanjiCodeName[input_kanji_encode_type]);
 
-#if defined(__FREEBSD__)
+#if defined(__FREEBSD__) || defined(__SUNOS__)
         if(iconv(ic, (const char**)&ptr_in, &bufsz_in, &ptr_out, &bufsz_out) < 0
 #else
         if(iconv(ic, &ptr_in, &bufsz_in, &ptr_out, &bufsz_out) < 0
