@@ -8,24 +8,14 @@
 #include <sys/wait.h>
 #include <sys/msg.h>
 #include <signal.h>
-
 #include <limits.h>
 
-#if defined(__DARWIN__)
-#include <sys/syslimits.h>
-#endif
-
 #include "saphire.h"
 #include "saphire_inner.h"
 
 //////////////////////////////////////////////////////////
 // 宣言
 //////////////////////////////////////////////////////////
-string_obj* gAncestalMemoryPipeIn;
-    // トップレベルの内部コマンド用のパイプ
-string_obj* gAncestalMemoryPipeOut;
-    // トップレベルの内部コマンド用のパイプ
-
 char gTmpDir[PATH_MAX];
     // 一時ファイルの置き場
 
@@ -138,12 +128,6 @@ hash_obj* gReadBuffers;
     // 内容 sRFd*
     // read用のファイルのリードバッファー
 
-vector_obj* gGlobalPipes;
-    // スタッカブルグローバルパイプのデータ
-
-vector_obj* gQGlobalPipes;
-    // キューグローバルパイプの読み込みんだデータ
-
 string_obj* gGlobalPipe;
     // グローバルパイプの読み込みんだデータ
 
@@ -166,10 +150,6 @@ hash_obj* gRegexsIM;
     // 正規表現のキャッシュ(ignore case|multi line)
     // キーは正規表現の文字列
     // 内容はregex_t
-hash_obj* gFWriteBuffers;
-    // 書き込みバッファ
-    // キーはファイルディスクリプタ番号の文字列
-    // 内容は書き込んだファイルの内容
 
 hash_obj* gInnerCommands;
     // 外部登録の内部コマンド
@@ -288,8 +268,6 @@ void err_msg(char* msg, char* sname, int line)
     snprintf(tmp, strlen(sname) + 128 + strlen(msg), "%s %d: %s\n", sname,  line, msg);
 
     string_put(gErrMsg, tmp);
-
-    FREE(tmp);
 }
 
 // コマンド名からコマンドIDを探す
@@ -310,41 +288,6 @@ void null_fun()
 {
 }
 
-// ポインターが差す文字列がリダイレクトかどうか
-BOOL parse_is_redirect(char* p)
-{
-    if(*p == '1' && *(p+1) == '>') {
-        return TRUE;
-    }
-    else if(*p == '2' && *(p+1) == '>') {
-        return TRUE;
-    }
-    else if(*p == '>') {
-        return TRUE;
-    }
-    else if(*p == '<') {
-        return TRUE;
-    }
-
-    return FALSE;
-}
-
-// ポインタが差す文字列がリダイレクトかどうか2
-BOOL parse_is_redirect2(char* p)
-{
-    return *p == '2' && *(p+1) == '>' && *(p+2) == '&' && *(p+3) == '1'
-        || *p == '&' && *(p+1) == '>';
-}
-
-// 変数で使える文字かどうか
-BOOL is_env_name_char(char c)
-{
-    return c >= 'a' && c <= 'z' || c >= 'A' && c <= 'Z' 
-            || c >= '0' && c <= '9'
-            || c == '_';
-}
-
-
 // 文字列をクォートして返す
 void saphire_get_quoted_fname(char* fname, string_obj* quoted_fname)
 {
@@ -388,7 +331,6 @@ void saphire_set_global_var(char* name, char* value)
     string_obj* global = hash_item(gGlobals, name);
 
     if(global) {
-        string_delete(global);
         hash_erase(gGlobals, name);
     }
     hash_put(gGlobals, name, STRING_NEW(value));
@@ -422,11 +364,6 @@ void saphire_set_array(char* name, vector_obj* array)
     vector_obj* v = hash_item(gArrays, name);
 
     if(v) {
-        int i;
-        for(i=0; i<vector_size(v); i++) {
-            string_delete(vector_item(v, i));
-        }
-        vector_delete(v);
         hash_erase(gArrays, name);
     }
 
@@ -456,7 +393,6 @@ void saphire_delete_local_var(char* name)
     string_obj* var = hash_item(top_stack, name);
 
     if(var) {
-        string_delete(var);
         hash_erase(top_stack, name);
     }
 
@@ -466,7 +402,6 @@ void saphire_delete_local_var(char* name)
     string_obj* var_size = hash_item(gGlobals, name_size);
 
     if(var_size) {
-        string_delete(var_size);
         hash_erase(gGlobals, name_size);
     }
 }
@@ -816,15 +751,6 @@ sJob* sJob_new()
     return self;
 }
 
-void sJob_delete(sJob* self)
-{
-    string_delete(self->mName);
-    vector_delete(self->mPIDs);
-    if(self->mTty) FREE(self->mTty);
-
-    FREE(self);
-}
-
 /////////////////////////////////////////////////////////
 // リダイレクト
 /////////////////////////////////////////////////////////
@@ -846,12 +772,6 @@ sRedirect* sRedirect_new2(sRedirect* redirect)
     return self;
 }
 
-void sRedirect_delete(sRedirect* self)
-{
-    string_delete(self->mFName);
-    FREE(self);
-}
-
 void sRedirect_save(sRedirect* self, int fd)
 {
     unsigned char uc;
@@ -919,185 +839,10 @@ sRedirect* sRedirect_load(int fd)
     }
     buf[n] = 0;
     self->mFName = STRING_NEW(buf);
-    FREE(buf);
     
     return self;
 }
 
-
-sRedirectTmp* sRedirectTmp_new(enum eRedirect type, int fd)
-{
-    sRedirectTmp* self = (sRedirectTmp*)MALLOC(sizeof(sRedirectTmp));
-    self->mType = type;
-    self->mFd = fd;
-
-    return self;
-}
-
-void sRedirectTmp_delete(sRedirectTmp* self)
-{
-    FREE(self);
-}
-
-sVar* sVar_new(MANAGED string_obj* name, MANAGED string_obj* index, MANAGED string_obj* index2, MANAGED string_obj* delimiter)
-{
-    sVar* self = MALLOC(sizeof(sVar));
-    self->mName = name;
-    self->mIndex = index;
-    self->mIndex2 = index2;
-    self->mDelimiter = delimiter;
-    return self;
-}
-
-sVar* sVar_new2(sVar* var)
-{
-    sVar* self = MALLOC(sizeof(sVar));
-    self->mName = STRING_NEW(string_c_str(var->mName));
-    if(var->mIndex) {
-        self->mIndex = STRING_NEW(string_c_str(var->mIndex));
-    }
-    else {
-        self->mIndex = NULL;
-    }
-    if(var->mIndex2) {
-        self->mIndex2 = STRING_NEW(string_c_str(var->mIndex2));
-    }
-    else {
-        self->mIndex2 = NULL;
-    }
-    self->mDelimiter = STRING_NEW(string_c_str(var->mDelimiter));
-    return self;
-}
-
-void sVar_delete(sVar* self)
-{
-    string_delete(self->mName);
-    if(self->mIndex) string_delete(self->mIndex);
-    if(self->mIndex2) string_delete(self->mIndex2);
-    string_delete(self->mDelimiter);
-
-    FREE(self);
-}
-
-sVar* sVar_load(int fd)
-{
-    sVar* self = MALLOC(sizeof(sVar));
-    int n;
-    char* buf;
-
-    if(read(fd, &n, sizeof(int)) < 0) {
-        perror("read");
-        exit(1);
-    }
-    
-    buf = MALLOC(n+1);
-    if(read(fd, buf, n) < 0) {
-        perror("read");
-        exit(1);
-    }
-    buf[n] = 0;
-    self->mName = STRING_NEW(buf);
-    FREE(buf);
-
-    if(read(fd, &n, sizeof(int)) < 0) {
-        perror("read");
-        exit(1);
-    }
-    
-    buf = MALLOC(n+1);
-    if(read(fd, buf, n) < 0) {
-        perror("read");
-        exit(1);
-    }
-    buf[n] = 0;
-    self->mIndex = STRING_NEW(buf);
-    FREE(buf);
-
-    if(read(fd, &n, sizeof(int)) < 0) {
-        perror("read");
-        exit(1);
-    }
-    
-    buf = MALLOC(n+1);
-    if(read(fd, buf, n) < 0) {
-        perror("read");
-        exit(1);
-    }
-    buf[n] = 0;
-    self->mIndex2 = STRING_NEW(buf);
-    FREE(buf);
-
-    if(read(fd, &n, sizeof(int)) < 0) {
-        perror("read");
-        exit(1);
-    }
-    
-    buf = MALLOC(n+1);
-    if(read(fd, buf, n) < 0) {
-        perror("read");
-        exit(1);
-    }
-    buf[n] = 0;
-    self->mDelimiter = STRING_NEW(buf);
-    FREE(buf);
-
-    return self;
-}
-
-void sVar_save(sVar* self, int fd)
-{
-    string_obj* str = self->mName;
-    int n = string_size(str);
-
-    if(write(fd, &n, sizeof(int)) < 0) {
-        perror("write");
-        exit(1);
-    }
-
-    if(write(fd, string_c_str(str), n) < 0) {
-        perror("write");
-        exit(1);
-    }
-
-    str = self->mIndex;
-    n = string_size(str);
-
-    if(write(fd, &n, sizeof(int)) < 0) {
-        perror("write");
-        exit(1);
-    }
-    if(write(fd, string_c_str(str), n) < 0) {
-        perror("write");
-        exit(1);
-    }
-
-    str = self->mIndex2;
-    n = string_size(str);
-
-    if(write(fd, &n, sizeof(int)) < 0) {
-        perror("write");
-        exit(1);
-    }
-
-    if(write(fd, string_c_str(str), n) < 0) {
-        perror("write");
-        exit(1);
-    }
-
-    str = self->mDelimiter;
-    n = string_size(str);
-
-    if(write(fd, &n, sizeof(int)) < 0) {
-        perror("write");
-        exit(1);
-    }
-
-    if(write(fd, string_c_str(str), n) < 0) {
-        perror("write");
-        exit(1);
-    }
-}
-
 sArg* sArg_new(int kind, void* body)
 {
     sArg* self = MALLOC(sizeof(sArg));
@@ -1118,18 +863,10 @@ sArg* sArg_new2(sArg* arg)
         self->mBody = STRING_NEW(string_c_str(arg->mBody));
         break;
 
-    case 1:
-        self->mBody = sVar_new2(arg->mBody);
-        break;
-
     case 2:
         self->mBody = sStatments_new2(arg->mBody);
         break;
 
-    case 3:
-        self->mBody = sVar_new2(arg->mBody);
-        break;
-
     case 4:
         self->mBody = sStatments_new2(arg->mBody);
         break;
@@ -1137,42 +874,15 @@ sArg* sArg_new2(sArg* arg)
     case 5:
         self->mBody = sBlock_new2(arg->mBody);
         break;
+
+    case 6:
+        self->mBody = sStatments_new2(arg->mBody);
+        break;
     }
 
     return self;
 }
 
-void sArg_delete(sArg* self)
-{
-    switch(self->mKind) {
-    case 0:
-        string_delete(self->mBody);
-        break;
-
-    case 1:
-        sVar_delete(self->mBody);
-        break;
-
-    case 2:
-        sStatments_delete(self->mBody);
-        break;
-
-    case 3:
-        sVar_delete(self->mBody);
-        break;
-
-    case 4:
-        sStatments_delete(self->mBody);
-        break;
-
-    case 5:
-        sBlock_delete(self->mBody);
-        break;
-    }
-
-    FREE(self);
-}
-
 void sArg_save(sArg* self, int fd)
 {
     int n = self->mKind;
@@ -1198,18 +908,10 @@ void sArg_save(sArg* self, int fd)
             }
             break;
 
-        case 1:
-            sVar_save(self->mBody, fd);
-            break;
-
         case 2:
             sStatments_save(self->mBody, fd);
             break;
 
-        case 3:
-            sVar_save(self->mBody, fd);
-            break;
-
         case 4:
             sStatments_save(self->mBody, fd);
             break;
@@ -1217,6 +919,10 @@ void sArg_save(sArg* self, int fd)
         case 5:
             sBlock_save(self->mBody, fd);
             break;
+
+        case 6:
+            sStatments_save(self->mBody, fd);
+            break;
     }
 }
 
@@ -1246,22 +952,13 @@ sArg* sArg_load(int fd)
         buf[m] = 0;
 
         self->mBody = STRING_NEW(buf);
-        FREE(buf);
         }
         break;
 
-    case 1:
-        self->mBody = sVar_load(fd);
-        break;
-
     case 2:
         self->mBody = sStatments_load(fd);
         break;
 
-    case 3:
-        self->mBody = sVar_load(fd);
-        break;
-
     case 4:
         self->mBody = sStatments_load(fd);
         break;
@@ -1269,6 +966,10 @@ sArg* sArg_load(int fd)
     case 5:
         self->mBody = sBlock_load(fd);
         break;
+
+    case 6:
+        self->mBody = sStatments_load(fd);
+        break;
     }
 
     return self;
@@ -1278,55 +979,6 @@ sArg* sArg_load(int fd)
 // 変数
 //////////////////////////////////////////////////////////
 
-//////////////////////////////////////////////////////////
-// コマンドライン引数
-//////////////////////////////////////////////////////////
-sArgTmp* sArgTmp_new(int kind, void* body)
-{
-    sArgTmp* self = MALLOC(sizeof(sArgTmp));
-
-    self->mKind = kind;
-    self->mBody = body;
-
-    return self;
-}
-
-void sArgTmp_delete(sArgTmp* self)
-{
-    switch(self->mKind)
-    {
-    case 0:
-        string_delete(self->mBody);
-        break;
-
-    case 1:
-        if(self->mBody) sVar_delete(self->mBody);
-        break;
-
-    case 2:
-        sRedirectTmp_delete(self->mBody);
-        break;
-
-    case 3:
-        if(self->mBody) sStatments_delete(self->mBody);
-        break;
-
-    case 4:
-        if(self->mBody) sVar_delete(self->mBody);
-        break;
-
-    case 5:
-        if(self->mBody) sStatments_delete(self->mBody);
-        break;
-
-    case 6:
-        if(self->mBody) sBlock_delete(self->mBody);
-        break;
-    }
-
-    FREE(self);
-}
-
 //////////////////////////////////////////////////////////
 // sSubshell
 //////////////////////////////////////////////////////////
@@ -1348,13 +1000,6 @@ sSubshell* sSubshell_new2(sSubshell* subshell)
     return self;
 }
 
-void sSubshell_delete(sSubshell* self)
-{
-    sStatments_delete(self->contents);
-
-    FREE(self);
-}
-
 void sSubshell_save(sSubshell* self, int fd)
 {
     sStatments_save(self->contents, fd);
@@ -1397,14 +1042,6 @@ sTry* sTry_new2(sTry* try_)
     return self;
 }
 
-void sTry_delete(sTry* self)
-{
-    sStatments_delete(self->loops);
-    sStatments_delete(self->loops2);
-
-    FREE(self);
-}
-
 void sTry_view(sTry* self)
 {
     sStatments_view(self->loops);
@@ -1450,14 +1087,6 @@ sWhile* sWhile_new2(sWhile* while_)
     return self;
 }
 
-void sWhile_delete(sWhile* self)
-{
-    sStatments_delete(self->joukensiki);
-    sStatments_delete(self->contents);
-
-    FREE(self);
-}
-
 void sWhile_view(sWhile* self)
 {
     sStatments_view(self->joukensiki);
@@ -1511,22 +1140,6 @@ sIf* sIf_new2(sIf* if_)
     return self;
 }
 
-void sIf_delete(sIf* self)
-{
-    int i;
-    for(i=0; i<vector_size(self->joukensiki_list); i++) {
-        sStatments_delete(vector_item(self->joukensiki_list, i));
-    }
-    vector_delete(self->joukensiki_list);
-
-    for(i=0; i<vector_size(self->contents_list); i++) {
-        sStatments_delete(vector_item(self->contents_list, i));
-    }
-    vector_delete(self->contents_list);
-
-    FREE(self);
-}
-
 void sIf_view(sIf* self)
 {
     int i;
@@ -1600,7 +1213,6 @@ sCommand* sCommand_new()
     sCommand* self = MALLOC(sizeof(sCommand));
 
     self->mKind = kCommand;
-    self->mArgTmps = VECTOR_NEW(50);
     self->mArgs = VECTOR_NEW(50);
 
     self->mRedirects = VECTOR_NEW(10);
@@ -1617,7 +1229,6 @@ sCommand* sCommand_new2(sCommand* command)
     sCommand* self = MALLOC(sizeof(sCommand));
 
     self->mKind = command->mKind;
-    self->mArgTmps = VECTOR_NEW(50);
 
     self->mArgs = VECTOR_NEW(50);
     int i;
@@ -1765,7 +1376,6 @@ sCommand* sCommand_load(int fd)
     int i;
     
     sCommand* self = MALLOC(sizeof(sCommand));
-    self->mArgTmps = VECTOR_NEW(30);
     
     /// mKind ///
     if(read(fd, &uc, sizeof(unsigned char)) < 0) {
@@ -1825,50 +1435,6 @@ sCommand* sCommand_load(int fd)
     return self;
 }
 
-void sCommand_delete(sCommand* self)
-{
-    int i;
-
-    for(i=0; i<vector_size(self->mArgTmps); i++) {
-        sArgTmp_delete(vector_item(self->mArgTmps, i));
-    }
-    vector_delete(self->mArgTmps);
-
-    for(i=0; i<vector_size(self->mArgs); i++) {
-        sArg_delete(vector_item(self->mArgs, i));
-    }
-    vector_delete(self->mArgs);
-
-    for(i=0; i<vector_size(self->mRedirects); i++) {
-        sRedirect_delete(vector_item(self->mRedirects, i));
-    }
-    vector_delete(self->mRedirects);
-
-    switch(self->mKind) {
-    case kIf:
-        sIf_delete(self->mExtra);
-        break;
-
-    case kWhile:
-        sWhile_delete(self->mExtra);
-        break;
-
-    case kTry:
-        sTry_delete(self->mExtra);
-        break;
-
-    case kSubshell:
-        sSubshell_delete(self->mExtra);
-        break;
-
-    case kDef:
-        sFunction_delete(self->mExtra);
-        break;
-    }
-
-    FREE(self);
-}
-
 /////////////////////////////////////////////////////////
 // 文 ls -al | grep aaa
 /////////////////////////////////////////////////////////
@@ -1888,6 +1454,8 @@ sStatment* sStatment_new()
     self->mGlobalPipeIn = FALSE;
     self->mGlobalPipeOut = FALSE;
 
+    self->mSTDINPipe = FALSE;
+
     return self;
 }
 
@@ -1918,6 +1486,8 @@ sStatment* sStatment_new2(sStatment* statment)
     self->mGlobalPipeIn = statment->mGlobalPipeIn;
     self->mGlobalPipeOut = statment->mGlobalPipeOut;
 
+    self->mSTDINPipe = statment->mSTDINPipe;
+
     return self;
 }
 
@@ -1954,22 +1524,8 @@ void sStatment_view(sStatment* self)
 
     printf("self->mGlobalPipeIn %d\n", self->mGlobalPipeIn);
     printf("self->mGlobalPipeOut %d\n", self->mGlobalPipeOut);
-}
 
-void sStatment_delete(sStatment* self)
-{
-    int i;
-    for(i=0; i<vector_size(self->mCommands); i++) {
-        sCommand_delete(vector_item(self->mCommands, i));
-    }
-    vector_delete(self->mCommands);
-
-    string_delete(self->mFName);
-    string_delete(self->mTitle);
-
-    if(self->mNotEvaled) { string_delete(self->mNotEvaled); }
-
-    FREE(self);
+    printf("self->mSTDINPipe %d\n", self->mSTDINPipe);
 }
 
 void sStatment_save(sStatment* self, int fd)
@@ -2076,6 +1632,13 @@ void sStatment_save(sStatment* self, int fd)
         perror("write");
         exit(1);
     }
+
+    // mSTDINPipe //
+    n = self->mSTDINPipe;
+    if(write(fd, &n, sizeof(int)) < 0) {
+        perror("write");
+        exit(1);
+    }
 }
 
 sStatment* sStatment_load(int fd)
@@ -2114,7 +1677,6 @@ sStatment* sStatment_load(int fd)
     }
     buf[n] = 0;
     self->mFName = STRING_NEW(buf);
-    FREE(buf);
     
     if(read(fd, &n, sizeof(int)) < 0) {
         perror("read");
@@ -2133,7 +1695,6 @@ sStatment* sStatment_load(int fd)
     }
     buf[n] = 0;
     self->mTitle = STRING_NEW(buf);
-    FREE(buf);
     
     if(read(fd, &n, sizeof(int)) < 0) {
         perror("read");
@@ -2154,7 +1715,6 @@ sStatment* sStatment_load(int fd)
         }
         buf[n] = 0;
         self->mNotEvaled = STRING_NEW(buf);
-        FREE(buf);
     }
     
     if(read(fd, &n, sizeof(int)) < 0) {
@@ -2187,6 +1747,12 @@ sStatment* sStatment_load(int fd)
     }
     self->mGlobalPipeOutNum = n;
 
+    if(read(fd, &n, sizeof(int)) < 0) {
+        perror("read");
+        exit(1);
+    }
+    self->mSTDINPipe = n;
+
     return self;
 }
 
@@ -2261,16 +1827,6 @@ sStatments* sStatments_load(int fd)
     return self;
 }
 
-void sStatments_delete(sStatments* self)
-{
-    int i;
-    for(i=0; i<vector_size(self->mStatments); i++) {
-        sStatment_delete(vector_item(self->mStatments, i));
-    }
-    vector_delete(self->mStatments);
-
-    FREE(self);
-}
 
 /////////////////////////////////////////////////////////
 // 関数
@@ -2304,19 +1860,6 @@ sFunction* sFunction_new2(sFunction* func)
     return self;
 }
 
-void sFunction_delete(sFunction* self)
-{
-    string_delete(self->name);
-    string_delete(self->arg_name);
-
-    if(self->statments) sStatments_delete(self->statments);
-        // NULLのときはdeleteしない
-        // NULLになるときはgFuncsに登録したとき
-        // statmentsはgFuncsの中に入りこっちではdeleteしない
-
-    FREE(self);
-}
-
 void sFunction_save(sFunction* self, int fd)
 {
     int n = string_length(self->name);
@@ -2359,7 +1902,6 @@ sFunction* sFunction_load(int fd)
     buf[n] = 0;
 
     self->name = STRING_NEW(buf);
-    FREE(buf);
 
     if(read(fd, &n, sizeof(int)) < 0) {
         perror("read");
@@ -2374,7 +1916,6 @@ sFunction* sFunction_load(int fd)
     buf[n] = 0;
 
     self->arg_name = STRING_NEW(buf);
-    FREE(buf);
 
     self->statments = sStatments_load(fd);
 
@@ -2388,27 +1929,6 @@ void sFunction_view(sFunction* self)
 }
 
 
-/////////////////////////////////////////////////////////
-// run_wait_cprog で使う
-/////////////////////////////////////////////////////////
-vector_obj* gStrayCProgs;
-
-sStrayCProg* sStrayCprog_new(int pid, int status, int pgroup)
-{
-    sStrayCProg* self = MALLOC(sizeof(sStrayCProg));
-    
-    self->mPid = pid;
-    self->mStatus = status;
-    self->mPGroup = pgroup;
-    
-    return self;
-}
-
-void sStrayCProg_delete(sStrayCProg* self)
-{
-    FREE(self);
-}
-
 ///////////////////////////////////////////////////////
 // 子プロセス
 ///////////////////////////////////////////////////////
@@ -2423,13 +1943,6 @@ sChildProgram* sChildProgram_new(pid_t pid, char* title, BOOL writer_process)
     return self;
 }
 
-void sChildProgram_delete(sChildProgram* self)
-{
-    string_delete(self->mTitle);
-
-    FREE(self);
-}
-
 //////////////////////////////////////////////////////
 // saphireが組み込まれたアプリケーションから
 // 登録された内部コマンド
@@ -2444,11 +1957,6 @@ sInnerCommand* sInnerCommand_new(char* name, fInnerCommand fun)
     return self;
 }
 
-void sInnerCommand_delete(sInnerCommand* self)
-{
-    string_delete(self->mName);
-    FREE(self);
-}
 
 /////////////////////////////////////////////////////////
 // プログラム名補完候補を再読み込み
@@ -2465,13 +1973,10 @@ static void saphire_rehash_without_user_fun()
 {
     hash_obj* check = HASH_NEW(3000);
 
-    int i;
-    for(i=0; i<vector_size(gSaphireProgCompletions); i++) {
-        string_delete(vector_item(gSaphireProgCompletions, i));
-    }
     vector_clear(gSaphireProgCompletions);
 
     /// 内部コマンド ///
+    int i;
     for(i=0; i<kCommand; i++) {
         vector_add(gSaphireProgCompletions, STRING_NEW(gStatmentKindStrs[i]));
     }
@@ -2571,9 +2076,6 @@ static void saphire_rehash_without_user_fun()
     }
 
     (void)vector_sort(gSaphireProgCompletions, name_sort);
-
-    hash_delete(check);
-
 }
 
 static void saphire_rehash_user_fun()
@@ -2610,12 +2112,10 @@ static void read_rc_file(BOOL run_user_runtimescript)
             fprintf(stderr, "%s", string_c_str(gErrMsg));
             exit(1);
         }
-        sRFd_delete(pipein);
         if(!sWFd_flash(pipeout)) {
             fprintf(stderr, "can't flash rc");
             exit(1);
         }
-        sWFd_delete(pipeout);
     }
     else {
         fprintf(stderr, "can't find %s file\n", rc_fname);
@@ -2639,9 +2139,6 @@ static void read_rc_file(BOOL run_user_runtimescript)
                     fprintf(stderr, "can't flash rc");
                     exit(1);
                 }
-                sRFd_delete(pipein);
-
-                sWFd_delete(pipeout);
             }
         }
     }
@@ -2665,8 +2162,6 @@ static void read_rc_obj_file(BOOL run_user_runtimescript)
             fprintf(stderr, "can't flash rc");
             exit(1);
         }
-        sRFd_delete(pipein);
-        sWFd_delete(pipeout);
     }
     else {
         fprintf(stderr, "can't find %s file\n", rc_fname);
@@ -2687,8 +2182,6 @@ static void read_rc_obj_file(BOOL run_user_runtimescript)
                     exit(1);
                 }
                 sWFd_flash(pipeout);
-                sWFd_delete(pipeout);
-                sRFd_delete(pipein);
             }
         }
     }
@@ -2699,8 +2192,6 @@ static void read_rc_obj_file(BOOL run_user_runtimescript)
 ////////////////////////////////////////////////////////
 void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript runtime_script, BOOL run_user_runtimescript)
 {
-    gGlobalPipes = VECTOR_NEW(10);
-    gQGlobalPipes = VECTOR_NEW(10);
     gGlobalPipe = STRING_NEW("");
     int i;
     for(i=0; i<10; i++) {
@@ -2709,8 +2200,6 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
 
     // ナンバーグローバルパイプの読み込みんだデータ
 
-    gAncestalMemoryPipeIn = STRING_NEW("");
-    gAncestalMemoryPipeOut = STRING_NEW("");
     gAppType = app_type;
     gJobControl = job_control;
 
@@ -2724,7 +2213,7 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
 
     gReadBuffers = HASH_NEW(100);
 
-    setenv("SAPHIRE_VERSION", "3.0.3", 1);
+    setenv("SAPHIRE_VERSION", "3.0.6", 1);
     setenv("SAPHIRE_PROMPT", "> ", 1);
     setenv("SAPHIRE_DOCDIR", DOCDIR, 1);
     char* home = getenv("HOME");
@@ -2747,10 +2236,7 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
     gRegexsI = HASH_NEW(100);
     gRegexsM = HASH_NEW(100);
     gRegexsIM = HASH_NEW(100);
-    gFWriteBuffers = HASH_NEW(512);
     
-    gStrayCProgs = VECTOR_NEW(10);
-
     gStackFrame = VECTOR_NEW(5);
     vector_add(gStackFrame, HASH_NEW(30));
 
@@ -2800,158 +2286,37 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
 
 void saphire_final()
 {
-    int i;
-    for(i=0; i<vector_size(gStackTraceFName); i++) {
-        string_delete(vector_item(gStackTraceFName, i));
-    }
-    vector_delete(gStackTraceFName);
-    vector_delete(gStackTraceLineNum);
-
-    int len = vector_size(gGlobalPipes);
-    for(i=0; i<len; i++) {
-        string_delete(vector_item(gGlobalPipes, i));
-    }
-    vector_delete(gGlobalPipes);
-
-    len = vector_size(gQGlobalPipes);
-    for(i=0; i<len; i++) {
-        string_delete(vector_item(gQGlobalPipes, i));
-    }
-    vector_delete(gQGlobalPipes);
-
-    string_delete(gGlobalPipe);
-    for(i=0; i<10; i++) {
-        string_delete(gGlobalPipeNum[i]);
-    }
-
-    hash_it* it = hash_loop_begin(gFuncs);
-    while(it != NULL) {
-        sFunction_delete(hash_loop_item(it));
-        it = hash_loop_next(it);
-    }
-    hash_delete(gFuncs);
-
-    for(i=0; i<vector_size(gStackFrame); i++) {
-        hash_obj* hash = vector_item(gStackFrame, i);
-
-        it = hash_loop_begin(hash);
-        while(it != NULL) {
-            string_delete(hash_loop_item(it));
-            it = hash_loop_next(it);
-        }
-        hash_delete(hash);
-    }
-    vector_delete(gStackFrame);
-
-    it = hash_loop_begin(gGlobals);
-    while(it != NULL) {
-        string_delete(hash_loop_item(it));
-        it = hash_loop_next(it);
-    }
-    hash_delete(gGlobals);
-
-    it = hash_loop_begin(gArrays);
-    while(it != NULL) {
-        vector_obj* v = hash_loop_item(it);
-
-        int i;
-        for(i=0; i<vector_size(v); i++) {
-            string_delete(vector_item(v, i));
-        }
-        vector_delete(v);
-
-        it = hash_loop_next(it);
-    }
-    hash_delete(gArrays);
-
-    it = hash_loop_begin(gHashs);
-    while(it != NULL) {
-        hash_obj* h = hash_loop_item(it);
-
-        hash_it* it2 = hash_loop_begin(h);
-        while(it2 != NULL) {
-            string_delete(hash_loop_item(it2));
-            it2 = hash_loop_next(it2);
-        }
-        hash_delete(h);
-
-        it = hash_loop_next(it);
-    }
-    hash_delete(gHashs);
-
-    for(i=0; i<vector_size(gJobs); i++) {
-        sJob_delete(vector_item(gJobs, i));
-    }
-    vector_delete(gJobs);
-
-    it = hash_loop_begin(gReadBuffers);
+    hash_it* it = hash_loop_begin(gReadBuffers);
     while(it) {
         sRFd* fd = (sRFd*)hash_loop_item(it);
         sRFd_close(fd);
-        sRFd_delete(fd);
         it = hash_loop_next(it);
     }
-    hash_delete(gReadBuffers);
     
-    for(i=0; i<vector_size(gSaphireProgCompletions); i++) {
-        string_delete(vector_item(gSaphireProgCompletions, i));
-    }
-    vector_delete(gSaphireProgCompletions);
-
     it = hash_loop_begin(gRegexs);
     while(it != NULL) {
         onig_free(hash_loop_item(it));
         it = hash_loop_next(it);
     }
-    hash_delete(gRegexs);
 
     it = hash_loop_begin(gRegexsI);
     while(it != NULL) {
         onig_free(hash_loop_item(it));
         it = hash_loop_next(it);
     }
-    hash_delete(gRegexsI);
 
     it = hash_loop_begin(gRegexsM);
     while(it != NULL) {
         onig_free(hash_loop_item(it));
         it = hash_loop_next(it);
     }
-    hash_delete(gRegexsM);
 
     it = hash_loop_begin(gRegexsIM);
     while(it != NULL) {
         onig_free(hash_loop_item(it));
         it = hash_loop_next(it);
     }
-    hash_delete(gRegexsIM);
     
-    it = hash_loop_begin(gFWriteBuffers);
-    while(it != NULL) {
-        string_delete(hash_loop_item(it));
-        it = hash_loop_next(it);
-    }
-    hash_delete(gFWriteBuffers);
-    
-    for(i=0; i<vector_size(gStrayCProgs); i++) {
-        sStrayCProg_delete(vector_item(gStrayCProgs, i));
-    }
-    vector_delete(gStrayCProgs);
-
-    it = hash_loop_begin(gInnerCommands);
-    while(it != NULL) {
-        sInnerCommand_delete(hash_loop_item(it));
-        it = hash_loop_next(it);
-    }
-    hash_delete(gInnerCommands);
-
-    for(i=0; i<vector_size(gDirStack); i++) {
-        string_delete(vector_item(gDirStack, i));
-    }
-    vector_delete(gDirStack);
-
-    string_delete(gErrMsg);
-
     /// プロセス置換で使った一時ファイルを掃除 ///
     int k;
     for(k=0; k<vector_size(gPSubTmpFiles); k++) {
@@ -2961,18 +2326,7 @@ void saphire_final()
             perror("unlink");
             exit(1);
         }
-
-        string_delete(fname);
     }
-    vector_delete(gPSubTmpFiles);
-
-    string_delete(gAncestalMemoryPipeIn);
-    string_delete(gAncestalMemoryPipeOut);
-
-    for(i=0; i<vector_size(gSaphireVarCompletions); i++) {
-        string_delete(vector_item(gSaphireVarCompletions, i));
-    }
-    vector_delete(gSaphireVarCompletions);
 }
 
 /////////////////////////////////////////////////////////////////////////
@@ -2998,12 +2352,6 @@ sRFd* sRFd_new2(int fd, char* str)
     return self;
 }
 
-void sRFd_delete(sRFd* self)
-{
-    string_delete(self->mBuffer);
-    FREE(self);
-}
-
 BOOL sRFd_close(sRFd* self)
 {
     if(self->mFd == -1) {
@@ -3556,12 +2904,6 @@ sWFd* sWFd_new(int fd)
     return self;
 }
 
-void sWFd_delete(sWFd* self)
-{
-    FREE(self->mBuffer);
-    FREE(self);
-}
-
 BOOL sWFd_close(sWFd* self)
 {
     if(self->mFd != -1) {
@@ -3602,7 +2944,6 @@ BOOL sWFd_push_back(sWFd* self, char* str)
         }
         *self->mP = 0;
          
-        FREE(old_buffer);
     }
 
     if(self->mP - self->mBuffer > BUFSIZ) {
@@ -3639,7 +2980,7 @@ BOOL sWFd_flash(sWFd* self)
                     }
     //printf("self->mFd %d\n", self->mFd);
                     perror("write(sWFd_flash)2");
-                    exit(1);
+                    return FALSE;
                 }
             }
 
@@ -3664,12 +3005,6 @@ vector_obj* gSaphireVarCompletions;               // 環境変数補完候補
 
 void saphire_make_var_completion_list()
 {
-    int i;
-    for(i=0; i<vector_size(gSaphireVarCompletions); i++) {
-        string_delete(vector_item(gSaphireVarCompletions, i));
-    }
-    vector_clear(gSaphireVarCompletions);
-
     /// 環境変数 ///
 #if HAVE_DECL_ENVIRON
     extern char **environ;
@@ -3759,19 +3094,6 @@ sBlock* sBlock_new2(sBlock* block)
     return self;
 }
 
-void sBlock_delete(sBlock* self)
-{
-    sStatments_delete(self->mStatments);
-    int i;
-    const int max = vector_size(self->mArgs);
-    for(i=0; i<max; i++) {
-        string_delete(vector_item(self->mArgs, i));
-    }
-    vector_delete(self->mArgs);
-
-    FREE(self);
-}
-
 sBlock* sBlock_load(int fd)
 {
     sBlock* self = MALLOC(sizeof(sBlock));
@@ -3799,7 +3121,6 @@ sBlock* sBlock_load(int fd)
         }
         buf[n] = 0;
         string_obj* buf2 = STRING_NEW(buf);
-        FREE(buf);
         
         vector_add(self->mArgs, buf2);
     }
